<?PHP

/*
   class.phpwikibot.php - main class of phpWikiBot
   
   $Id: class.phpwikibot.php,v 1.7 2006-02-28 01:09:17 kb Exp $

   This file is part of phpWikiBot
   Copyright (C) 2006 Krzysztof Burghardt.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

include 'diff.php';

class phpWikiBot {
	private $login;
	private $password;
	private $wiki;
	private $response;
	private $rawArticle;
	private $articleExists;
	private $phpWikiBotVersion;
	private $wpStarttime, $wpEdittime, $wpEditToken;
	private $userAgent;

	public function __construct($wiki) {
		$curlVersion = curl_version();
		$this->phpWikiBotVersion = 'phpWikiBot 0.1.0 (PHP ' . phpversion() . ' / cURL ' . $curlVersion['version'] . ')';
		$this->wiki = $wiki;
		$this->userAgent = 'Mozilla/5.0 (X11; U; Linux x86_64; pl-PL; rv:1.7.12) Gecko/20050924 Firefox/1.0.7 (Debian package 1.0.7-1)';
	}
	public function __destruct() {
		unset($this->login);
		unset($this->password);
		unset($this->wiki);
		unset($this->response);
		unset($this->rawArticle);
		unset($this->articleExists);
		unset($this->phpWikiBotVersion);
		unset($this->wpStarttime);
		unset($this->wpEdittime);
		unset($this->wpEditToken);
	}

	private function parseResponse() {
		preg_match('/HTTP\/1.1 (.+)/', $this->response, $matches);
		$httpResponseCode = $matches[1];
		if ($httpResponseCode == 302) {
		    preg_match('/Location: (.+)/', $this->response, $matches);
		    echo '302 - ' . $matches[1] . "\n";
		    return;
		}
		echo $httpResponseCode . "\n";
	}
	private function curlStatistics($ch) {
		$s = curl_getinfo($ch);
		echo '[ Time: ' . number_format($s['total_time'], 2, '.', '') .
		    ' s, Upload: ' . number_format($s['size_upload']/1000, 2, '.', '') .
		    ' kB @ ' . number_format($s['speed_upload']/1000, 2, '.', '') .
		    ' kB/s, Download: ' . number_format($s['size_download']/1000, 2, '.', '') .
		    ' kB @ ' . number_format($s['speed_download']/1000, 2, '.', '') . " kB/s ]\n";
	}
	public function login($login, $password) {
		$this->login = $login;
		$this->password = $password;
		echo 'Requesting login as ' . $this->login . ' to ' . $this->wiki . '... ';
		
		$ch = curl_init('http://' . $this->wiki . '/w/index.php?title=Specjalna:Userlogin&action=submitlogin&type=login&returnto=Strona_g%C5%82%C3%B3wna');
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
		curl_setopt($ch, CURLOPT_HEADER, TRUE);
		curl_setopt($ch, CURLOPT_COOKIEJAR, 'cookie.db');
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
		curl_setopt($ch, CURLOPT_POSTFIELDS, "wpPassword=$this->password&wpLoginattempt=Aanmelden%20%26%20Inschrijven&wpName=$this->login&wpRemember=1");
		curl_setopt($ch, CURLOPT_USERAGENT, $this->userAgent);
		$this->response = curl_exec($ch);
		if (curl_errno($ch)) {
		    echo curl_error($ch) . "\n";
		    sleep(10);
		    $this->login($login, $password);
		} else {
		    $this->parseResponse();
		    $this->curlStatistics($ch);
		}
		curl_close($ch);
	}
	private function getArticle($title) {
		echo "Requesting article $title... ";
		$ch = curl_init('http://' . $this->wiki . '/w/index.php?title=' . urlencode($title) . '&action=edit');
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
		curl_setopt($ch, CURLOPT_HEADER, TRUE);
		curl_setopt($ch, CURLOPT_COOKIEFILE, 'cookie.db');
		curl_setopt($ch, CURLOPT_POST, FALSE);
		curl_setopt($ch, CURLOPT_USERAGENT, $this->userAgent);
		$this->response = curl_exec($ch);
		if (curl_errno($ch)) {
		    echo curl_error($ch) . "\n";
		    sleep(10);
		    $this->getArticle($title);
		} else {
		    $this->parseResponse();
		    $this->curlStatistics($ch);
		}
		curl_close($ch);
	}
	public function checkIfArticleExists($title) {
		$this->getArticle($title);
		$this->parseArticleCookie();
		if (strlen($this->articleExists)==0) {
		    return false;
		} else {
		    return true;
		}
	}
	private function parseArticleCookie() {
		preg_match("/<li id=\"ca-history\"><a href=\"\/w\/index.php\?title=(.+)&amp;action=history/", $this->response, $matches);
		$this->articleExists = $matches[1];
		preg_match("/<input type='hidden' value=\"(.+)\" name=\"wpStarttime\"/", $this->response, $matches);
		$this->wpStarttime = $matches[1];
		preg_match("/<input type='hidden' value=\"(.+)\" name=\"wpEdittime\"/", $this->response, $matches);
		$this->wpEdittime = $matches[1];
		preg_match("/<input type='hidden' value=\"(.+)\" name=\"wpEditToken\"/", $this->response, $matches);
		$this->wpEditToken = $matches[1];
	}
	public function readArticle($title) {
		if ($this->checkIfArticleExists($title)) {
		    $this->parseArticleBody();
		    return $this->rawArticle;
		} else {
		    echo $title . ": article does not exists!\n";
		}
	}
	private function parseArticleBody() {
		preg_match("/<textarea[^>]*>(.+)<\/textarea>/s", $this->response, $matches);
		$this->rawArticle = $matches[1];
	}
	private function postArticle($title, $article, $summary, $minor) {
		echo "Posting article $title... ";
		$toPost = 'wpSave=1&wpSummary=';
		$toPost = $toPost . urlencode($summary);
		$toPost = $toPost . '&wpTextbox1=';
		$toPost = $toPost . urlencode($article);
		$toPost = $toPost . '&wpEdittime=' . $this->wpEdittime . '&wpStarttime=' . $this->wpStarttime . '&wpMinoredit=' . $minor . '&wpEditToken=' . $this->wpEditToken;

		$ch = curl_init('http://' . $this->wiki . '/w/index.php?title=' . urlencode($title) . '&action=submit');
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
		curl_setopt($ch, CURLOPT_HEADER, TRUE);
		curl_setopt($ch, CURLOPT_COOKIEFILE, 'cookie.db');
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $toPost);
		curl_setopt($ch, CURLOPT_USERAGENT, $this->userAgent);
		$this->response = curl_exec($ch);
		if (curl_errno($ch)) {
		    echo curl_error($ch) . "\n";
		    sleep(10);
		    $this->postArticle($title, $article, $summary, $minor);
		} else {
		    $this->parseResponse();
		    $this->curlStatistics($ch);
		}
		curl_close($ch);
	}
	public function createArticle($title, $article, $summary, $minor) {
		if (! $this->checkIfArticleExists($title)) {
		    echo "\n[[$title]]\n$article\n\n";
		    $this->postArticle($title, $article, $summary, $minor);
		} else {
		    echo urldecode($this->articleExists) . ": article exists, createArticle() will not touch it!\n";
		}
	}
	private function showDiff($article) {
		$old = preg_split ('/[\n]+/', $this->rawArticle);
		$new = preg_split ('/[\n]+/', $article);
		return arr_diff($old, $new). "\n";
	}
	public function editArticle($title, $article, $summary, $minor) {
		if ($this->checkIfArticleExists($title)) {
		    echo "\n[[$title]] diff:\n" . $this->showDiff($article);
		    $this->postArticle($title, $article, $summary, $minor);
		} else {
		    echo urldecode($this->articleExists) . ": article does not exists, use createArticle()!\n";
		}
	}

}

?>
